@@ -1,22 +1,33 @@
 import itertools
 import random
-from typing import (Any, Iterable, List, Literal, Optional,
-                    Sequence, Tuple, TypeVar)
+from typing import (
+    Any,
+    Iterable,
+    List,
+    Literal,
+    Optional,
+    Sequence,
+    Tuple,
+    Union,
+)
+
 from transformers.tokenization_utils_base import PreTrainedTokenizerBase
 
-from ..base import BaseMapper, TransformElementType
+from ..base.mapper import BatchedBaseMapper, SingleBaseMapper
+from ..base.types import TransformElementType
 
 
-LabelType = TypeVar('LabelType', int, float)
+class TokensSequencesPaddingMapper(SingleBaseMapper):
+    bos: List[int]
+    sep: List[int]
+    eos: List[int]
 
-
-class TokensSequencesPaddingMapper(BaseMapper):
     def __init__(
         self,
         tokenizer: PreTrainedTokenizerBase,
-        input_field: str = 'input_ids',
+        input_field: str = "input_ids",
     ) -> None:
-        """ Mapper that add BOS/SEP/EOS sequences of tokens.
+        """Mapper that add BOS/SEP/EOS sequences of tokens.
 
         Args:
             tokenizer (PreTrainedTokenizerBase): Tokenizer to use for
@@ -24,24 +35,24 @@ class TokensSequencesPaddingMapper(BaseMapper):
             input_field (str, optional): The field to add special tokens to.
                 Defaults to 'input_ids'.
         """
-        super().__init__()
-
-        self.input_fields = [input_field]
-        self.output_fields = [input_field]
+        super().__init__(
+            input_fields=[input_field], output_fields=[input_field]
+        )
         self.bos, self.sep, self.eos = self._find_special_token_ids(tokenizer)
-        self.batched = False
 
     @staticmethod
     def _find_special_token_ids(
-        tokenizer: PreTrainedTokenizerBase
-    ) -> Tuple[Sequence[int], Sequence[int], Sequence[int]]:
+        tokenizer: PreTrainedTokenizerBase,
+    ) -> Tuple[List[int], List[int], List[int]]:
         """By default, tokenizers only know how to concatenate 2 fields
         as input; However, for our purposes, we might care about more than
         just 2. This function tries to figure out the best strategy by
         tokenizing two fake sequences and selecting beginning, mid, and
         end sequence(s) tokens."""
 
-        bos, sep, eos = [], [], []
+        bos: List[int] = []
+        sep: List[int] = []
+        eos: List[int] = []
 
         class FirstFakeSequenceSymbol(int):
             ...
@@ -75,9 +86,9 @@ class TokensSequencesPaddingMapper(BaseMapper):
         seqs_count = len(sequences)
 
         padded_sequences = [
-            (self.bos if i == 0 else []) +
-            seq +
-            (self.eos if (i + 1) == seqs_count else self.sep)
+            (self.bos if i == 0 else [])
+            + seq
+            + (self.eos if (i + 1) == seqs_count else self.sep)
             for i, seq in enumerate(sequences)
         ]
         data[self.input_fields[0]] = padded_sequences
@@ -86,9 +97,11 @@ class TokensSequencesPaddingMapper(BaseMapper):
 
 
 class AttentionMaskSequencePaddingMapper(TokensSequencesPaddingMapper):
-    def __init__(self,
-                 tokenizer: PreTrainedTokenizerBase,
-                 input_field: str = 'attention_mask') -> None:
+    def __init__(
+        self,
+        tokenizer: PreTrainedTokenizerBase,
+        input_field: str = "attention_mask",
+    ) -> None:
         """Mapper to add BOS/SEP/EOS tokens to an attention mask sequence.
 
         Args:
@@ -106,9 +119,11 @@ class AttentionMaskSequencePaddingMapper(TokensSequencesPaddingMapper):
 
 
 class TokenTypeIdsSequencePaddingMapper(TokensSequencesPaddingMapper):
-    def __init__(self,
-                 tokenizer: PreTrainedTokenizerBase,
-                 input_field: str = 'token_type_ids') -> None:
+    def __init__(
+        self,
+        tokenizer: PreTrainedTokenizerBase,
+        input_field: str = "token_type_ids",
+    ) -> None:
         """Mapper to add BOS/SEP/EOS tokens to a token type ids sequence.
 
         Args:
@@ -123,12 +138,20 @@ class TokenTypeIdsSequencePaddingMapper(TokensSequencesPaddingMapper):
         sequences = data[self.input_fields[0]]
         seqs_count = len(sequences)
         padded_sequences = [
-            # a sequence start with BOS tags or SEP tags
-            ([i for _ in self.bos] if i == 0 else [i for _ in self.sep]) +
-            seq +
-            # a sequence ends with EOS tags or nothing if it is not
-            # the last sequence
-            ([i for _ in self.eos] if (i + 1) == seqs_count else [])
+            (
+                # a sequence start with BOS tags or SEP tags
+                [i for _ in self.bos]
+                if i == 0
+                else [i for _ in self.sep]
+            )
+            + seq
+            + (
+                # a sequence ends with EOS tags or nothing if it is not
+                # the last sequence
+                [i for _ in self.eos]
+                if (i + 1) == seqs_count
+                else []
+            )
             for i, seq in enumerate(sequences)
         ]
         data[self.input_fields[0]] = padded_sequences
@@ -136,10 +159,12 @@ class TokenTypeIdsSequencePaddingMapper(TokensSequencesPaddingMapper):
         return data
 
 
-class MakeAttentionMaskMapper(BaseMapper):
-    def __init__(self,
-                 input_field: str = 'input_ids',
-                 output_field: str = 'attention_mask') -> None:
+class MakeAttentionMaskMapper(SingleBaseMapper):
+    def __init__(
+        self,
+        input_field: str = "input_ids",
+        output_field: str = "attention_mask",
+    ) -> None:
         """Mapper to create attention masks from input ids.
 
         Args:
@@ -148,9 +173,9 @@ class MakeAttentionMaskMapper(BaseMapper):
             output_field (str, optional): The name of the field containing
                 the attention mask. Defaults to 'attention_mask'.
         """
-        super().__init__()
-        self.input_fields = [input_field]
-        self.output_fields = [output_field]
+        super().__init__(
+            input_fields=[input_field], output_fields=[output_field]
+        )
 
     def transform(self, data: TransformElementType) -> TransformElementType:
         sequences = data[self.input_fields[0]]
@@ -159,13 +184,15 @@ class MakeAttentionMaskMapper(BaseMapper):
         return data
 
 
-class LabelsMaskerMapper(BaseMapper):
-    def __init__(self,
-                 labels_field: str = 'labels',
-                 strategy: Literal['all', 'one', 'sample'] = 'all',
-                 sample_prob: Optional[float] = None,
-                 label_mask_id: LabelType = -100) -> None:
-        """ Given a sequence of labels, this mapper will mask some of them.
+class LabelsMaskerMapper(BatchedBaseMapper):
+    def __init__(
+        self,
+        labels_field: str = "labels",
+        strategy: Literal["all", "one", "sample"] = "all",
+        sample_prob: Optional[float] = None,
+        label_mask_id: Union[int, float] = -100,
+    ) -> None:
+        """Given a sequence of labels, this mapper will mask some of them.
         Useful when wanting to create more samples by masking a subset of
         labels, or when running evaluations and want to predict one label at
         the time.
@@ -186,28 +213,26 @@ class LabelsMaskerMapper(BaseMapper):
                 label. Defaults to -100.
 
         """
-        super().__init__()
-        self.input_fields = [labels_field]
-        self.output_fields = [labels_field]
-        self.batched = True
+        super().__init__(
+            input_fields=[labels_field], output_fields=[labels_field]
+        )
 
-        if strategy not in ['all', 'one', 'sample']:
-            raise ValueError(f'Unknown strategy {strategy}')
-        elif strategy == 'random' and sample_prob is None:
-            raise ValueError('no `sample_prob` provided for `random` strategy')
-        elif strategy == 'one' and sample_prob is not None:
-            raise ValueError('Do not provide `sample_prob` for `one` strategy')
+        if strategy not in ["all", "one", "sample"]:
+            raise ValueError(f"Unknown strategy {strategy}")
+        elif strategy == "random" and sample_prob is None:
+            raise ValueError("no `sample_prob` provided for `random` strategy")
+        elif strategy == "one" and sample_prob is not None:
+            raise ValueError("Do not provide `sample_prob` for `one` strategy")
 
         self.strategy: str = strategy
         self.sample_prob: float = sample_prob or 0.0
-        self.label_mask_id: LabelType = label_mask_id
+        self.label_mask_id: Union[int, float] = label_mask_id
 
     def transform(
-        self,
-        data: Iterable[TransformElementType]
+        self, data: Iterable[TransformElementType]
     ) -> Iterable[TransformElementType]:
 
-        if self.strategy == 'all':
+        if self.strategy == "all":
             # there's no masking to do if the strategy is all!
             # `all` is provided for convenience, but it's not really a
             # transformation, just an identity function
@@ -218,7 +243,7 @@ class LabelsMaskerMapper(BaseMapper):
         for sample in data:
             labels = sample[labels_field_name]
 
-            if self.strategy == 'one':
+            if self.strategy == "one":
                 # make sequences of labels where only one label is
                 # not masked for each sequence; the goal is to transform
                 # n sequences with m active labels into n * m sequences
@@ -226,11 +251,12 @@ class LabelsMaskerMapper(BaseMapper):
                 for i, _ in enumerate(labels):
 
                     # new labels sequence here
-                    new_labels = [l if i == j else self.label_mask_id
-                                  for j, l in enumerate(labels)]
+                    new_labels = [
+                        l if i == j else self.label_mask_id
+                        for j, l in enumerate(labels)
+                    ]
 
-                    new_sample = {**sample,
-                                  **{labels_field_name: new_labels}}
+                    new_sample = {**sample, **{labels_field_name: new_labels}}
                     yield new_sample
 
                     # # add the new sample you just made to the batch
@@ -238,7 +264,7 @@ class LabelsMaskerMapper(BaseMapper):
                     # for f in other_fields_name:
                     #     new_data[f].append(data[f][sample_pos])
 
-            if self.strategy == 'sample':
+            if self.strategy == "sample":
                 expected_slice_size = max(
                     int(len(labels) * self.sample_prob), 1
                 )
@@ -246,28 +272,31 @@ class LabelsMaskerMapper(BaseMapper):
                 random.shuffle(labels_pos)
 
                 for active_pos in (
-                    labels_pos[i:i + expected_slice_size]
+                    labels_pos[i : i + expected_slice_size]
                     for i in range(0, len(labels_pos), expected_slice_size)
                 ):
                     # new labels sequence here
-                    new_labels = [l if i in active_pos else self.label_mask_id
-                                  for i, l in enumerate(labels)]
+                    new_labels = [
+                        l if i in active_pos else self.label_mask_id
+                        for i, l in enumerate(labels)
+                    ]
 
                     # add the new sample you just made to the batch,
                     # plus all extra fields
-                    new_sample = {**sample,
-                                  **{labels_field_name: new_labels}}
+                    new_sample = {**sample, **{labels_field_name: new_labels}}
                     yield new_sample
 
 
-class MultiSequenceStriderMapper(BaseMapper):
-    def __init__(self,
-                 max_stride_count: int,
-                 length_reference_field: str,
-                 max_length: Optional[int] = None,
-                 extra_length_per_seq: Optional[int] = None,
-                 tokenizer: Optional[PreTrainedTokenizerBase] = None,
-                 max_step: Optional[int] = None) -> None:
+class MultiSequenceStriderMapper(BatchedBaseMapper):
+    def __init__(
+        self,
+        max_stride_count: int,
+        length_reference_field: str,
+        max_length: Optional[int] = None,
+        extra_length_per_seq: Optional[int] = None,
+        tokenizer: Optional[PreTrainedTokenizerBase] = None,
+        max_step: Optional[int] = None,
+    ) -> None:
         """Mapper to create multiple subset sequences from a single sequence
         of sequences.
 
@@ -295,12 +324,13 @@ class MultiSequenceStriderMapper(BaseMapper):
             max_step (int, optional): Not used at the moment.
 
         """
-        super().__init__()
+        super().__init__(
+            input_fields=[length_reference_field],
+            output_fields=[length_reference_field],
+        )
 
-        self.input_fields = self.output_fields = [length_reference_field]
-        self.batched = True
         self.max_stride_count = max_stride_count
-        self.max_length = max_length or float('inf')
+        self.max_length = max_length or float("inf")
 
         if extra_length_per_seq is None:
             if tokenizer is not None:
@@ -309,8 +339,13 @@ class MultiSequenceStriderMapper(BaseMapper):
                 # padding of sequences. Because we don't know if a sequence
                 # will be at the beginning, middle, or end of the batch,
                 # we take the max length between the various separators
-                bos, sep, eos = TokensSequencesPaddingMapper.\
-                    _find_special_token_ids(tokenizer)
+                (
+                    bos,
+                    sep,
+                    eos,
+                ) = TokensSequencesPaddingMapper._find_special_token_ids(
+                    tokenizer
+                )
                 extra_length_per_seq = 2 * max(len(bos), len(sep), len(eos))
             else:
                 # if extra length is not provided, we simply assume that
@@ -324,8 +359,7 @@ class MultiSequenceStriderMapper(BaseMapper):
             raise NotImplementedError("max_step is not supported yet")
 
     def transform(
-        self,
-        data: Iterable[TransformElementType]
+        self, data: Iterable[TransformElementType]
     ) -> Iterable[TransformElementType]:
 
         ref_field_name, *_ = self.input_fields
@@ -336,8 +370,8 @@ class MultiSequenceStriderMapper(BaseMapper):
 
             for seq_pos_end in range(len(sample[ref_field_name])):
                 current_seq_length = (
-                    len(sample[ref_field_name][seq_pos_end]) +
-                    self.extra_length_per_seq
+                    len(sample[ref_field_name][seq_pos_end])
+                    + self.extra_length_per_seq
                 )
 
                 if current_seq_length > self.max_length:
@@ -347,17 +381,17 @@ class MultiSequenceStriderMapper(BaseMapper):
                     )
 
                 stride_too_long = (
-                    (cumulative_stride_length + current_seq_length)
-                    > self.max_length
-                )
+                    cumulative_stride_length + current_seq_length
+                ) > self.max_length
                 stride_has_too_many_seqs = (
-                    (seq_pos_end - seq_pos_start)
-                    >= self.max_stride_count
-                )
+                    seq_pos_end - seq_pos_start
+                ) >= self.max_stride_count
 
                 if stride_too_long or stride_has_too_many_seqs:
-                    yield {k: v[seq_pos_start:seq_pos_end]
-                           for k, v in sample.items()}
+                    yield {
+                        k: v[seq_pos_start:seq_pos_end]
+                        for k, v in sample.items()
+                    }
 
                     cumulative_stride_length = 0
                     seq_pos_start = seq_pos_end
@@ -371,16 +405,19 @@ class MultiSequenceStriderMapper(BaseMapper):
 
             if len(out[ref_field_name]) < 1:
                 import ipdb
+
                 ipdb.set_trace()
             yield out
 
 
-class SingleValueToSequenceMapper(BaseMapper):
-    def __init__(self,
-                 single_value_field: str,
-                 like_field: str = 'input_ids',
-                 strategy: Literal['first', 'last', 'all'] = 'first',
-                 padding_id: LabelType = -100) -> None:
+class SingleValueToSequenceMapper(SingleBaseMapper):
+    def __init__(
+        self,
+        single_value_field: str,
+        like_field: str = "input_ids",
+        strategy: Literal["first", "last", "all"] = "first",
+        padding_id: Union[int, float] = -100,
+    ) -> None:
         """Mapper to create a sequence of values from single value.
         Useful when casting a sequence classification task to a sequence
         tagging task, e.g. making a prediction for a sequence of sentences
@@ -403,30 +440,29 @@ class SingleValueToSequenceMapper(BaseMapper):
                     of the new sequence; the padding_id will be ignored.
             padding_id: id to use for the padding token. Default is -100.
         """
-        super().__init__()
-
-        self.input_fields = [single_value_field, like_field]
-        self.output_fields = [single_value_field]
+        super().__init__(
+            input_fields=[single_value_field, like_field],
+            output_fields=[single_value_field],
+        )
         self.strategy = strategy
         self.padding_id = padding_id
-        self.batched = False
 
     def _make_sequence_from_value(
-        self,
-        value: LabelType,
-        like_seq: Sequence[Any]
-    ) -> Sequence[LabelType]:
+        self, value: Union[int, float], like_seq: Sequence[Any]
+    ) -> Sequence[Union[int, float]]:
 
-        if self.strategy == 'first':
-            return ([value] +       # type: ignore
-                    [self.padding_id for _ in range(len(like_seq) - 1)])
-        elif self.strategy == 'last':
-            return ([self.padding_id for _ in range(len(like_seq) - 1)] +
-                    [value])        # type: ignore
-        elif self.strategy == 'all':
+        if self.strategy == "first":
+            return [value] + [
+                self.padding_id for _ in range(len(like_seq) - 1)
+            ]
+        elif self.strategy == "last":
+            return [self.padding_id for _ in range(len(like_seq) - 1)] + [
+                value
+            ]
+        elif self.strategy == "all":
             return [value for _ in like_seq]
         else:
-            raise ValueError(f'Strategy {self.strategy} is not supported')
+            raise ValueError(f"Strategy {self.strategy} is not supported")
 
     def transform(self, data: TransformElementType) -> TransformElementType:
         labels_field_name, like_field_name, *_ = self.input_fields
@@ -434,26 +470,30 @@ class SingleValueToSequenceMapper(BaseMapper):
         data[labels_field_name] = [
             self._make_sequence_from_value(
                 value=label, like_seq=data[like_field_name][i]
-            ) for i, label in enumerate(data[labels_field_name])
+            )
+            for i, label in enumerate(data[labels_field_name])
         ]
         return data
 
 
-class SequencesConcatenateMapper(BaseMapper):
+class SequencesConcatenateMapper(SingleBaseMapper):
     def __init__(self, concat_fields: Optional[List[str]] = None):
-        super().__init__()
-
+        super().__init__(
+            input_fields=concat_fields, output_fields=concat_fields
+        )
         self.concat_fields = (
             set(concat_fields) if concat_fields is not None else None
         )
-        self.input_fields = concat_fields or []
-        self.output_fields = concat_fields or []
-        self.batched = False
 
     def _to_concat(self, field_name: str) -> bool:
         return self.concat_fields is None or field_name in self.concat_fields
 
     def transform(self, data: TransformElementType) -> TransformElementType:
-        return {key: (list(itertools.chain.from_iterable(value))
-                      if self._to_concat(key) else value)
-                for key, value in data.items()}
+        return {
+            key: (
+                list(itertools.chain.from_iterable(value))
+                if self._to_concat(key)
+                else value
+            )
+            for key, value in data.items()
+        }
